class OrderBook {

	constructor(options) {
		this.options = options;
		this.orderbook = {Asks: [], Bids: []};
	}

	get() {
		return this.orderbook;
	}

	build(orderbook) {
		if (!orderbook) return;
		if (!orderbook.Time) orderbook.Time = Date.now();
		let { Asks, Bids } = orderbook;
		this.orderbook = {Asks:[], Bids:[]};
		this.update(Asks, Bids, orderbook.Time);
	}

	/**
	 * make sure bids is descending and asks is ascending
	 */
	update(asks, bids, time = null) {

		let Asks = [];
		let Bids = [];
		if (!time) time = Date.now();

		let minAskPrice = null;
		if (asks[0]) {
			if (asks[0].Amount > 0) minAskPrice = asks[0].Price;
			for (let i = 1; i < asks.length; i++) {
				if (!minAskPrice && asks[i].Amount > 0) minAskPrice = asks[i].Price;
				if (asks[i - 1].Price >= asks[i].Price) {
					console.log(asks);
					throw new Error('orderbook.update: asks should be ascending array');
				}
			}
		}
		
		let maxBidPrice = null;
		if (bids[0]) {
			if (bids[0].Amount > 0) maxBidPrice = bids[0].Price;
			for (let i = 1; i < bids.length; i++) {
				if (!maxBidPrice && bids[i].Amount > 0) maxBidPrice = bids[i].Price;
				if (bids[i - 1].Price <= bids[i].Price) {
					console.log(bids);
					throw new Error('orderbook.update: bids should be descending array');
				}
			}
		}

		if (maxBidPrice && minAskPrice && minAskPrice <= maxBidPrice) {
			throw new Error('asks should greater than bids');
		}

		if (true) {
			if (!asks) asks = [];
			let oldIdx = 0;
			asks.map(ask => {
				let {Price, Amount} = ask;

				while ( oldIdx < this.orderbook.Asks.length && this.orderbook.Asks[oldIdx].Price < Price) {
					Asks.push(this.orderbook.Asks[oldIdx]);
					oldIdx++;
				}

				if (this.orderbook.Asks[oldIdx] && this.orderbook.Asks[oldIdx].Price === Price) {
					if (Amount > 0) {
						this.orderbook.Asks[oldIdx].Amount = Amount;
						Asks.push(this.orderbook.Asks[oldIdx]);
					}
					oldIdx++;
				} else {
					if (Amount > 0) {
						Asks.push({
							Price,
							Amount
						});
					}
				}
			});
			if (oldIdx < this.orderbook.Asks.length) {
				for ( let i = oldIdx; i < this.orderbook.Asks.length; i++) {
					Asks.push(this.orderbook.Asks[i]);
				}
			}
		}

		if (true) {
			if (!bids) bids = [];
			let oldIdx = 0;
			bids.map(bid => {
				let {Price, Amount} = bid;

				while ( oldIdx < this.orderbook.Bids.length && this.orderbook.Bids[oldIdx].Price > Price) {
					Bids.push(this.orderbook.Bids[oldIdx]);
					oldIdx++;
				}

				if (this.orderbook.Bids[oldIdx] && this.orderbook.Bids[oldIdx].Price === Price) {
					if (Amount > 0) {
						this.orderbook.Bids[oldIdx].Amount = Amount;
						Bids.push(this.orderbook.Bids[oldIdx]);
					}
					oldIdx++;
				} else {
					if (Amount > 0) {
						Bids.push({
							Price,
							Amount
						});
					}
				}
			});
			if (oldIdx < this.orderbook.Bids.length) {
				for ( let i = oldIdx; i < this.orderbook.Bids.length; i++) {
					Bids.push(this.orderbook.Bids[i]);
				}
			}
		}

		if (maxBidPrice !== null) {
			Asks = Asks.filter(t => t.Price >= maxBidPrice);
		}

		if (minAskPrice !== null) {
			Bids = Bids.filter(t => t.Price < minAskPrice);
		}

		this.orderbook.Asks = Asks;
		this.orderbook.Bids = Bids;
		this.orderbook.Time = time;
	}

}


module.exports = OrderBook;